﻿using System.IO;
using System.Threading;
using System.Threading.Tasks;
using Luski.net.Structures;
using Luski.net.Structures.Public;
using Luski.Shared.PublicServers.V1.Enums;

namespace Luski.net.Interfaces;

/// <summary>
/// Represents the curent user
/// </summary>
public interface IUser
{
    /// <summary>
    /// The current Id of the user
    /// </summary>
    long Id { get; }
    /// <summary>
    /// The cerrent username of the user
    /// </summary>
    string DisplayName { get; }
    /// <summary>
    /// The current status of the user
    /// </summary>
    UserStatus Status { get; }

    /// <summary>
    /// The color of the display name
    /// </summary>
    Task<Color> GetColor();
    /// <summary>
    /// will returen the picture type of the user
    /// </summary>
    PictureType PictureType { get; }
    /// <summary>
    /// Gets the current avatar of the user
    /// </summary>
    Task<Stream> GetAvatar(CancellationToken CancellationToken);
    /// <summary>
    /// Gets the current user keys
    /// </summary>
    /// <returns></returns>
    Task<PublicKeyInfo[]> GetUserKeys(CancellationToken CancellationToken);

}
