using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using Luski.net.Classes;
using Luski.net.Enums;
using Luski.net.Interfaces;
using Luski.net.JsonTypes;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;

namespace Luski.net.Structures.Public;

public class SocketUser : IUser
{
    public PublicServer Server { get; init; } = default!;
    public long Id { get; init; } = default!;
    public string DisplayName { get; init; } = default!;
    public virtual UserStatus Status { get; internal set; } = default!;
    public PictureType PictureType { get; init; } = default!;
    public long[] RoleIds { get; init; } = default!;
    private List<Role>? RawRoles = null;
    
    public async Task<Color> GetColor()
    {
        return (await GetRoles())[0].Color;
    }

    public async Task<Role[]> GetRoles()
    {
        if (RawRoles is null)
        {
            RawRoles = new();
            foreach (long r in RoleIds)
            {
                Role f = await Server.GetRole(r);
                RawRoles.Add(f);
            }
            RawRoles.Sort(new RoleComparer());
        }
        
        return RawRoles.ToArray();
    }
    
    public async Task<Stream> GetAvatar(CancellationToken CancellationToken)
    {
        bool isc = System.IO.File.Exists(Server.Storage.GetStorageDirectory(StorageDirectory.Avatars) + Id.ToString());
        if (!isc) await Server.GetFromServer($"socketuserprofile/Avatar/{Id}", Server.Storage.GetStorageDirectory(StorageDirectory.Avatars) + Id.ToString(), CancellationToken);
        return Server.Storage.GetResourceStream(StorageDirectory.Avatars, Id.ToString());
    }

    public Task<PublicKeyInfo[]> GetUserKeys(CancellationToken CancellationToken)
    {
        KeysGetSTC data = Server.GetFromServer($"Keys/UserKeys/{Id}", KeysGetSTCContext.Default.KeysGetSTC, CancellationToken).Result;
        List<PublicKeyInfo> pki = new();
        foreach (KeyGetSTC key in data.Keys)
        {
            pki.Add(new()
            {
                Id = key.ID,
                Owner = key.Owner,
                EncryptionType = key.EncryptionType,
                Data = Convert.FromBase64String(key.Data)
            });
        }
        return Task.FromResult(pki.ToArray());
    }
}